# Script for Figures & Examples from AEA 2011, Demonstration Session 409

#==============================================================================
# CITATION
# Pierce, S. J. (2011, November). Master Teacher Series: Using equivalence
#   tests to prove that groups don't differ: How to generate "evidence of
#   absence" rather than "absence of evidence". Demonstration session presented
#   at Evaluation 2011: Values and Valuing in Evaluation, the annual conference
#   of the American Evaluation Association, Anaheim, CA.

#==============================================================================
# REVISION HISTORY:
# 10/01/2011: Steve started this file. Created Figure 1 (treatment vs. control).
# 10/09/2011: Added Figure 2 (pre-test/post-test).
# 10/23/2011: Added code for my.panel.Dotplot() and Figure 3.
# 10/24/2011: Continued work on Figure 3, edited various comments.

#==============================================================================
# SHOW USEFUL INFORMATION.

start.time <- Sys.time() # Store the current time
start.time # Display the current time

citation() # Show info about the version of R in use.

memory.limit() # Show how much memory (in MB) R has available.

memory.size() # Show how much memory (in MB) R is using.

#==============================================================================
# SET THE WORKING FOLDER. This tells R where to look for files and where to
# save them by default.

setwd("C:/Users/Steve/Documents/Professional/Presentations/AEA 2011")

#==============================================================================
# Load packages.

library(lattice)        # Really nice graphics tools.
library(effects)        # To easily extract CIs from lm() model fits.
library(latticeExtra)   # For useful add-ons to lattice like as.layer()
library(Hmisc)          # for xYplot() and Dotplot()
library(equivalence)    # for tost()
library(foreign)        # for write.foreign()

# Set lattice package options to default to producing black & white plots
lattice.options(default.theme = canonical.theme(color = FALSE))

#==============================================================================
# Figure 1: GRAPH SHOWING A CONTROL VS. TREATMENT COMPARISON.

# Generate some simulated data.

set.seed(3539288)
F1.df <- data.frame(Group = sort(rep(c("Control","Treatment"), times = 20)),
                    Score = c(rnorm(20, mean = 50, sd = 10),
                              rnorm(20, mean = 85, sd = 10)))

# Use a one-way ANOVA to analyze the data

F1.lm <- lm(Score ~ Group, data = F1.df)
anova(F1.lm, test = "F")

# Extract the means and confidence intervals.

F1.s <- summary(effect("Group", F1.lm))
F1.CIs <- data.frame(Group = c("Control","Treatment"),
                     Group2 = c(0, 1),
                     Score = as.numeric(F1.s$effect),
                     LL = as.numeric(F1.s$lower),
                     UL = as.numeric(F1.s$upper))

# Open a 3" x 3" graphics window (actual size to paste into PowerPoint presentation).

trellis.device(width = 3, height = 3)

# Create an object to hold some settings that minimize the white space padding
# around the plot.

my.padding <- list(top.padding = 0, main.key.padding = 0, key.axis.padding = 0,
                   axis.ylab.padding = 0, ylab.key.padding  = 0, right.padding = 0,
                   left.padding = 0, key.sub.padding = 0, bottom.padding = 0,
                   axis.right = 0, key.right = 0)

# Create the basic bar chart, then layer on the confidence intervals. The plus
# sign after the barchart() is what signals another layer is coming.

barchart(Score ~ Group, data=F1.CIs, ylim = c(0, 110), scales = list(cex = 1.2),
         ylab = list("Test Score", cex = 1.3),
         par.settings = list(layout.heights = my.padding,
		             layout.widths = my.padding)) +
as.layer(segplot(Group ~ LL + UL, data = F1.CIs, centers = Score, draw.bands = F,
                 ylab = list("Test Score", cex = 1.3), scales = list(cex = 1.2),
                 segments.fun=panel.arrows, ends = "both", angle = 90, length = 1.5,
                 unit = "mm", horizontal = F, cex = 1.1, lwd = 2, ylim = c(0, 110),
                 par.settings = list(layout.heights = my.padding,
	                             layout.widths = my.padding)))

#==============================================================================
# FIGURE 2: GRAPH SHOWING A PRE-TEST VS. POST-TEST COMPARISON.

# Generate some simulated data.

set.seed(5648131)
F2.df <- data.frame(Time = sort(rep(c("Pre-Test","Post-Test"), times = 20)),
                    Score = c(rnorm(20, mean = 25, sd = 5),
                              rnorm(20, mean = 12, sd = 5)))
F2.df$Time <- ordered(F2.df$Time, levels = c("Pre-Test","Post-Test"))

# Use a one-way ANOVA to analyze the data

F2.lm <- lm(Score ~ Time, data = F2.df)
anova(F2.lm, test = "F")

# Extract the means and confidence intervals.

F2.s <- summary(effect("Time", F2.lm))
F2.CIs <- data.frame(Time = ordered(c("Pre-Test","Post-Test"),
                                    levels = c("Pre-Test","Post-Test")),
                     Time2 = c(0, 1),
                     Score = as.numeric(F2.s$effect),
                     LL = as.numeric(F2.s$lower),
                     UL = as.numeric(F2.s$upper))

# Open a 3" x 3" graphics window (actual size to paste into PowerPoint presentation).

trellis.device(width = 3, height = 3)

# Create an object to hold some settings that minimize the white space padding
# around the plot.

my.padding <- list(top.padding = 0, main.key.padding = 0, key.axis.padding = 0,
                   axis.ylab.padding = 0, ylab.key.padding  = 0, right.padding = 0,
                   left.padding = 0, key.sub.padding = 0, bottom.padding = 0,
                   axis.right = 0, key.right = 0)

# Create the basic bar chart, then layer on the confidence intervals. The plus
# sign after the barchart() is what signals another layer is coming.

F2 <- xYplot(Cbind(Score, LL, UL) ~ Time2, data=F2.CIs, type = "b", cex = 1.1, pch = 16,
             xlim = c(-.25, 1.25), ylim = c(0, 30), lwd = 2, cap = .05,
             scales = list(cex = 1.2, alternating = c(1, 0),
                           x = list(at = c(0,1), labels = c("Pre-Test","Post-Test"))),
             main = "", xlab = "", ylab = list("Physical Functioning", cex = 1.3),
             par.settings = list(layout.heights = my.padding,
                                 layout.widths = my.padding),
             lattice.options = trellis.par.set(axis.components = list(top = list(tck = 0))))
print(F2)
		

#==============================================================================
# CUSTOM PANEL FUNCTION for the Dotplot() from Hmisc package.

# Customize the panel.Dotplot function to use vertical bars instead of + signs
# at the ends of the 95% CIs. Nearly all of this function is identical to
# panel.Dotplot as produced by the author of Hmisc package. Using the + signs
# make it look like the inner intervals are wider than they really should be.
# This section is here entirely because I'm just being very picky about how the
# graph I want to construct to illustrate the interpretation of TOST and t-test
# CIs will actually look.


my.panel.Dotplot <- function (x, y, groups = NULL, pch = dot.symbol$pch, col = dot.symbol$col,
                              cex = dot.symbol$cex, font = dot.symbol$font, abline, vert.lines, ...)
{
    gfun <- ordGridFun(.R.)
    segmnts <- gfun$segments
    y <- as.numeric(y)
    gp <- length(groups)
    dot.symbol <- trellis.par.get(if (gp)
        "superpose.symbol"
    else "dot.symbol")
    dot.line <- trellis.par.get("dot.line")
    plot.line <- trellis.par.get(if (gp)
        "superpose.line"
    else "plot.line")
    gfun$abline(h = unique(y), lwd = dot.line$lwd, lty = dot.line$lty,
        col = dot.line$col)
    if (!missing(abline)) {
        if (length(names(abline)))
            do.call("panel.abline", abline)
        else for (i in 1:length(abline)) do.call("panel.abline",
            abline[[i]])
    }
    other <- attr(x, "other")
    x <- oldUnclass(x)
    attr(x, "other") <- NULL
    if (length(other)) {
        nc <- ncol(other)
        segmnts(other[, 1], y, other[, nc], y, lwd = plot.line$lwd[1],
            lty = plot.line$lty[1], col = plot.line$col[1])
        if (nc == 4) {
            segmnts(other[, 2], y, other[, 3], y, lwd = plot.line$lwd[1],
                lty = plot.line$lty[1], col = plot.line$col[1])
            # Below here Steve modified the pch arcgument.
            gfun$points(other[, 2], y, pch = "|", pos = 3, cex = cex*1.5,
                        col = col, font = font)
            gfun$points(other[, 3], y, pch = "|", pos = 3, cex = cex*1.5,
                        col = col, font = font)
            # End of Steve's modifications.
        }
        if (gp)
            panel.superpose(x, y, groups = as.numeric(groups),
                pch = pch, col = col, cex = cex, font = font,
                ...)
        else gfun$points(x, y, pch = pch[1], cex = cex, col = col,
            font = font)
    }
    else {
        if (gp)
            panel.superpose(x, y, groups = as.numeric(groups),
                pch = pch, col = col, cex = cex, font = font,
                ...)
        else panel.dotplot(x, y, pch = pch, col = col, cex = cex,
            font = font, ...)
    }
    if (gp) {
        if (.R.)
            Key <- function(x = 0, y = 1, lev, cex, col, font,
                pch, other) {
                if (!length(x))
                  x <- 0.05
                if (!length(y))
                  y <- 0.95
                rlegendg(x, y, legend = lev, cex = cex, col = col,
                  pch = pch, other = other)
                invisible()
            }
        else Key <- function(x = NULL, y = NULL, lev, cex, col,
            font, pch, other) {
            if (length(x)) {
                if (is.list(x)) {
                  y <- x$y
                  x <- x$x
                }
                key(x = x, y = y, text = list(lev, col = col),
                  points = list(cex = cex, col = col, font = font,
                    pch = pch), transparent = TRUE)
            }
            else key(text = list(lev, col = col), points = list(cex = cex,
                col = col, font = font, pch = pch), transparent = TRUE)
            invisible()
        }
        lev <- levels(as.factor(groups))
        ng <- length(lev)
        formals(Key) <- list(x = NULL, y = NULL, lev = lev, cex = cex[1:ng],
            col = col[1:ng], font = font[1:ng], pch = pch[1:ng],
            other = NULL)
        storeTemp(Key)
    }
    # Steve added a piece to annotate the graph with vertical lines at
    # boundaries of margin of equivalence (-epsilon, epsilon) and at zero
    panel.abline(v = vert.lines, lty = 2, lwd = 2,
                 col = c("red", "blue", "red"))
}

#==============================================================================
# FIGURE 3: DOTPLOT OF SCENARIOS FOR EQUIVALENCE TEST VS. T-TEST.

#------------------------------------------------------------------------------
# Scenario 1: Simulated data for groups that are both equal & equivalent.

set.seed(5671894)
s1.df <- data.frame(Group = sort(rep(c("Control","Treatment"), times = 20)),
                    Score = c(rnorm(20, mean = 82, sd = 10),
                              rnorm(20, mean = 85, sd = 10)))

# Scenario 1: Standard t-test with alpha = .05 (95% CI).

s1.t <- t.test(x = s1.df$Score[1:20], y = s1.df$Score[21:40], var.equal = TRUE,
               paired = FALSE, conf.level = .95, alternative = "two.sided")
s1.t

# Scenario 1: TOST equivalence test with alpha = .05 (90% CI);
# margin of equivalence = epsilon = 15.

s1.tost <- tost(x = s1.df$Score[1:20], y = s1.df$Score[21:40], var.equal = TRUE,
                paired = FALSE, alpha = .05, epsilon = 15)
s1.tost

# Create a small data frame with results from Scenario 1
s.1 <- data.frame(Scenario = 1,
                  S.Label = "Equal + Equivalent",
                  Est = s1.tost$mean.diff,
                  LL.90 = s1.tost$ci.diff[1],
                  UL.90 = s1.tost$ci.diff[2],
                  LL.95 = s1.t$conf.int[1],
                  UL.95 = s1.t$conf.int[2])
s.1

#------------------------------------------------------------------------------
# Scenario 2: Simulated data for groups that are equal, but not equivalent.

set.seed(671926)
s2.df <- data.frame(Group = sort(rep(c("Control","Treatment"), times = 20)),
                    Score = c(rnorm(20, mean = 82, sd = 25),
                              rnorm(20, mean = 85, sd = 25)))

# Scenario 2: Standard t-test with alpha = .05 (95% CI).

s2.t <- t.test(x = s2.df$Score[1:20], y = s2.df$Score[21:40], var.equal = TRUE,
               paired = FALSE, conf.level = .95, alternative = "two.sided")
s2.t

# Scenario 2: TOST equivalence test with alpha = .05 (90% CI);
# margin of equivalence = epsilon = 15.

s2.tost <- tost(x = s2.df$Score[1:20], y = s2.df$Score[21:40], var.equal = TRUE,
                paired = FALSE, alpha = .05, epsilon = 15)
s2.tost

# Create a small data frame with results from Scenario 2
s.2 <- data.frame(Scenario = 2,
                  S.Label = "Equal, Not Equivalent",
                  Est = s2.tost$mean.diff,
                  LL.90 = s2.tost$ci.diff[1],
                  UL.90 = s2.tost$ci.diff[2],
                  LL.95 = s2.t$conf.int[1],
                  UL.95 = s2.t$conf.int[2])
s.2

#------------------------------------------------------------------------------
# Scenario 3: Simulated data for groups that are not equal, but equivalent.

set.seed(3274997)
s3.df <- data.frame(Group = sort(rep(c("Control","Treatment"), times = 20)),
                    Score = c(rnorm(20, mean = 75, sd = 10),
                              rnorm(20, mean = 85, sd = 10)))

# Scenario 3: Standard t-test with alpha = .05 (95% CI).

s3.t <- t.test(x = s3.df$Score[1:20], y = s3.df$Score[21:40], var.equal = TRUE,
               paired = FALSE, conf.level = .95, alternative = "two.sided")
s3.t

# Scenario 3: TOST equivalence test with alpha = .05 (90% CI);
# margin of equivalence = epsilon = 15.

s3.tost <- tost(x = s3.df$Score[1:20], y = s3.df$Score[21:40], var.equal = TRUE,
                paired = FALSE, alpha = .05, epsilon = 15)
s3.tost

# Create a small data frame with results from Scenario 3
s.3 <- data.frame(Scenario = 3,
                  S.Label = "Not Equal, But Equivalent",
                  Est = s3.tost$mean.diff,
                  LL.90 = s3.tost$ci.diff[1],
                  UL.90 = s3.tost$ci.diff[2],
                  LL.95 = s3.t$conf.int[1],
                  UL.95 = s3.t$conf.int[2])
s.3

#------------------------------------------------------------------------------
# Scenario 4: Simulated data for groups that are not equal, and not equivalent.

set.seed(5628719)
s4.df <- data.frame(Group = sort(rep(c("Control","Treatment"), times = 20)),
                    Score = c(rnorm(20, mean = 92, sd = 10),
                              rnorm(20, mean = 82, sd = 10)))

# Scenario 4: Standard t-test with alpha = .05 (95% CI).

s4.t <- t.test(x = s4.df$Score[1:20], y = s4.df$Score[21:40], var.equal = TRUE,
               paired = FALSE, conf.level = .95, alternative = "two.sided")
s4.t

# Scenario 4: TOST equivalence test with alpha = .05 (90% CI);
# margin of equivalence = epsilon = 15.

s4.tost <- tost(x = s4.df$Score[1:20], y = s4.df$Score[21:40], var.equal = TRUE,
                paired = FALSE, alpha = .05, epsilon = 15)
s4.tost

# Create a small data frame with results from Scenario 4
s.4 <- data.frame(Scenario = 4,
                  S.Label = "Not Equal, Not Equivalent",
                  Est = s4.tost$mean.diff,
                  LL.90 = s4.tost$ci.diff[1],
                  UL.90 = s4.tost$ci.diff[2],
                  LL.95 = s4.t$conf.int[1],
                  UL.95 = s4.t$conf.int[2])
s.4

#------------------------------------------------------------------------------
# Create and annotate Figure 3.

# Combine the four scenarios into a data frame for use in Figure 3.

F3.df <- rbind(s.1, s.2, s.3, s.4)

# Set parameters that control the tick marks & labeling on the axes.

my.scales <- list(alternating = F, cex = 1.2, tck = c(1, 0),
                  x = list(at = seq(from = -15, to = 15, by = 5)),
                  y = list(at = c(1, 2, 3, 4)))

# Set parameters that control how much padding space is used around various
# elements of the plot.

my.padding <- list(top.padding = 0, main.key.padding = 0, key.axis.padding = 0,
                   axis.ylab.padding = 0, ylab.key.padding  = 0, right.padding = 0,
                   left.padding = 0, key.sub.padding = 0, bottom.padding = 1,
                   axis.right = 0, key.right = 0)

# Open a 7" x 3.5" graphics window (actual size to paste into PowerPoint presentation).

trellis.device(width = 7, height = 3.5)

# Create Figure 3, then update it to use custom scales and panel function and
# add text labels for the margin of equivalence.

F3 <- Dotplot(Scenario ~ Cbind(Est, LL.95, LL.90, UL.90, UL.95), data=F3.df,
                ylab=list("Scenario", cex = 1.3), lwd = 2, cex = 1.25, pch = 19,
                xlab=list("90% & 95% CIs for (Control Mean - Treatment Mean)", cex = 1.3),
                ylim = c(0.5, 4.5), xlim = c(-20, 20),
                vert.lines = c(-epsilon, 0, epsilon),
                par.settings = list(layout.heights = my.padding,
		                            layout.widths = my.padding,
                                    plot.line = list(lwd = 2)))
update(F3, scales = my.scales, panel = my.panel.Dotplot)
ltext(x = 115, y = 255, labels = expression(-epsilon), cex = 1.75, col = "red")
ltext(x = 605, y = 255, labels = expression(+epsilon), cex = 1.75, col = "red")

#==============================================================================
# EXPORT THE 4 SCENARIOS TO A TEXT FILE FOR IMPORT INTO SPSS.

Scenarios <- cbind(Scenario = sort(rep(1:4, times = 40)),
                  rbind(s1.df, s2.df, s3.df, s4.df))
Scenarios$Group <- rep(sort(rep(c(0,1), times = 20)), times = 4)
Scenarios

write.foreign(df = Scenarios, datafile = "Scenarios.sav",
              codefile = "Import_Scenarios.sps", package = "SPSS")



#==============================================================================
# SHOW USEFUL INFORMATION.

sessionInfo()        # Display info about versions of loaded packages, etc.
citation("lattice")
citation("latticeExtra")
citation("effects")
citation("Hmisc")
citation("foreign")


# dev.off() # Close the PDF file.

end.time <- Sys.time() # Store the current time.
end.time               # Display the current time.

end.time - start.time # Show how long it took to run the whole script.
